from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.post_api_authn_saml_connector_id_body import PostApiAuthnSamlConnectorIdBody
from ...types import Response


def _get_kwargs(
    connector_id: str,
    *,
    body: PostApiAuthnSamlConnectorIdBody,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": f"/api/authn/saml/{connector_id}",
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == HTTPStatus.FOUND:
        return None
    if response.status_code == HTTPStatus.BAD_REQUEST:
        return None
    if response.status_code == HTTPStatus.NOT_FOUND:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    connector_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: PostApiAuthnSamlConnectorIdBody,
) -> Response[Any]:
    """SAML ACS endpoint (social)

     The Assertion Consumer Service (ACS) endpoint for Simple Assertion Markup Language (SAML) social
    connectors.

    SAML social connectors are deprecated. Use the SSO SAML connector instead.

    Args:
        connector_id (str):
        body (PostApiAuthnSamlConnectorIdBody): arbitrary

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        connector_id=connector_id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    connector_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: PostApiAuthnSamlConnectorIdBody,
) -> Response[Any]:
    """SAML ACS endpoint (social)

     The Assertion Consumer Service (ACS) endpoint for Simple Assertion Markup Language (SAML) social
    connectors.

    SAML social connectors are deprecated. Use the SSO SAML connector instead.

    Args:
        connector_id (str):
        body (PostApiAuthnSamlConnectorIdBody): arbitrary

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        connector_id=connector_id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)
